import Foundation
import Security
import ExpoModulesCore

struct GenericError: Error, LocalizedError {
  var errorDescription: String?
  init(_ message: String) {
    self.errorDescription = message
  }
}

// クライアント証明書を処理する URLSessionDelegate
class ClientCertificateDelegate: NSObject, URLSessionDelegate {
  let certificatePath: String?
  let certificatePassphrase: String?

  init(certificatePath: String?, certificatePassphrase: String?) {
    self.certificatePath = certificatePath
    self.certificatePassphrase = certificatePassphrase
  }

  func urlSession(_ session: URLSession,
                  didReceive challenge: URLAuthenticationChallenge,
                  completionHandler: @escaping (URLSession.AuthChallengeDisposition, URLCredential?) -> Void) {
    print("[START] ClientCertificateDelegate")
    print(challenge.protectionSpace.authenticationMethod)
    print(challenge.protectionSpace.authenticationMethod == NSURLAuthenticationMethodClientCertificate)

    guard challenge.protectionSpace.authenticationMethod == NSURLAuthenticationMethodClientCertificate else {
        completionHandler(.performDefaultHandling, nil)
        return
    }
    guard let certificatePath = certificatePath, let identity = retrieveClientIdentity(from: certificatePath, passphrase: certificatePassphrase) else {
        completionHandler(.performDefaultHandling, nil)
        return
    }
    print(identity)

    let credential = URLCredential(identity: identity, certificates: nil, persistence: .forSession)
    completionHandler(.useCredential, credential)
  }

  private func retrieveClientIdentity(from path: String, passphrase: String?) -> SecIdentity? {
    guard let certData = try? Data(contentsOf: URL(fileURLWithPath: path)) else {
        print("⚠️ 証明書ファイルを読み込めません: \(path)")
        return nil
    }
    var items: CFArray?
    let options: [String: Any] = [kSecImportExportPassphrase as String: passphrase ?? ""]
    let status = SecPKCS12Import(certData as CFData, options as CFDictionary, &items)

    guard status == errSecSuccess,
          let itemsArray = items as? [[String: Any]],
          let firstItem = itemsArray.first,
          let identity = firstItem[kSecImportItemIdentity as String] as! SecIdentity? else {
        print("⚠️ クライアント証明書のインポートに失敗")
        return nil
    }
    return identity
  }
}

// Expo Modules 用のリクエストパラメータ
struct RequestParams: Record {
  @Field var url: String
  @Field var method: String = "GET"
  @Field var headers: [String: String] = [:]
  @Field var body: String? = nil
  @Field var certificate: CertificateParams? = nil
}

struct CertificateParams: Record {
  @Field var path: String? = nil
  @Field var passphrase: String? = nil
}

public class ExpoHttpClientModule: Module {
  public func definition() -> ModuleDefinition {
    Name("ExpoHttpClient")

    AsyncFunction("performRequest") { (params: RequestParams) async throws -> Dictionary<String, Any> in
      guard let requestURL = URL(string: params.url) else {
        throw GenericError("Invalid URL provided: \(params.url)")
      }
      var request = URLRequest(url: requestURL)
      request.httpMethod = params.method

      for (key, value) in params.headers {
        request.addValue(value, forHTTPHeaderField: key)
      }
      if let body = params.body {
        request.httpBody = body.data(using: .utf8)
      }

      let delegate = ClientCertificateDelegate(certificatePath: params.certificate?.path, certificatePassphrase: params.certificate?.passphrase)
      let session = URLSession(configuration: .default, delegate: delegate, delegateQueue: nil)

      return try await withCheckedThrowingContinuation { continuation in
        let task = session.dataTask(with: request) { data, response, error in
          if let error = error {
            continuation.resume(throwing: GenericError("Network error: \(error.localizedDescription)"))
            return
          }

          guard let httpResponse = response as? HTTPURLResponse else {
            continuation.resume(throwing: GenericError("Invalid response from server"))
            return
          }
          let headers = httpResponse.allHeaderFields.reduce(into: [String: String]()) { dict, pair in
            if let key = pair.key as? String, let value = pair.value as? String {
              dict[key] = value
            }
          }
          let result: [String: Any] = [
            "status": httpResponse.statusCode,
            "statusText": HTTPURLResponse.localizedString(forStatusCode: httpResponse.statusCode),
            "headers": headers,
            "body": String(data: data ?? Data(), encoding: .utf8) ?? ""
          ]
          continuation.resume(returning: result)
        }
        task.resume()
      }
    }
  }
}
